//-------------------SpaßCAN Interface--------------//
//--------------------------------------------------//
//-----------------------SpaßCAN--------------------//
//------Copyright (C) 2016  Julian Zimmermann-------//
//==================================================//
//This program is free software;you can redistribute//
//it and/or modify it under the terms of the        //
//GNU General Public License as published by the    //
//Free Software Foundation; either version 3 of the //
//License, or (at your option) any later version.   //
//                                                  //
//This program is distributed in the hope that it   //
//will be useful, but WITHOUT ANY WARRANTY; without //
//even the implied warranty of MERCHANTABILITY or   //
//FITNESS FOR A PARTICULAR PURPOSE. See the GNU     //
//General Public License for more details.          //
//                                                  //
//You should have received a copy of the GNU General//
//Public License along with this program; if not,   //
//see <http://www.gnu.org/licenses/>.               //
//                                                  //
//==================================================//
//-----------------Versionshistory------------------//
//
//Version beta.1 (15.06.16)
// -CAN-Busanbindung
// -Serielle Anmeldung
// -Rückmelder werden angenommen und ausgegeben (001)
// -RFID Reader werden ausgegeben
//
//Version 1.0 (21.09.16)
// -Serieller RASCII-Befehlssatz (by RobertJ.Versluis)
// -Programmierung hinzu
//==================================================//


#include <mcp_can.h>
#include <mcp_can_dfs.h>
#include <SPI.h>
#include <RASCII.h>

MCP_CAN CAN(10);
RASCII Rascii(false);
//RASCII Rascii485(false);

int i,j,k = 0;
byte len = 0;
unsigned char buf[4];
unsigned char sendbuf[8];
unsigned long id = 0;
boolean angemeldet[256];
char serialbuf[16];

//----SUBROUTINEN----
void ANMELDEN (void) {
  angemeldet[id & 0xFF] = true;
  sendbuf[0] = id;                                        // Quittung, dass die Anmeldung erfolgt
  sendbuf[1] = 0x00;
  CAN.sendMsgBuf(0x700, 0, 0, 2, sendbuf);                // ID= 00, STD_FRAME, kein RTR_FRAME,len = 1,DATA = id,255,255
  return;
}
void setCV( byte* cmd ) {
  char s[32];
  sendbuf[0] = cmd[1];                                                  //Data0: Zieladresse
  sendbuf[1] = cmd[2];                                                  //Data1: Option
  sendbuf[2] = cmd[3];                                                  //Data2: Wert
  CAN.sendMsgBuf(0x700, 0, 0, 3, sendbuf);
  sprintf(s, "Prog: CV %d : %d", cmd[2], cmd[3]);
  Rascii.write(s);
}

void evaluateCmd(byte* cmd) {
char s[32];
    if( cmd[0] == CMD_INFO ) {
      sprintf(s, "SpaßCAN Interface V 1.0 addr=%d", 0x01);
      Rascii.write(s);
      sprintf(s, "This is free software");
      Rascii.write(s);
      sprintf(s, "and you are welcome to");
      Rascii.write(s);
      sprintf(s, "redistribute it under");
      Rascii.write(s);
      sprintf(s, "the conditions of GNU GPL");
      Rascii.write(s);
      sprintf(s, "http://www.gnu.org/licenses/");
      Rascii.write(s);
    }
    else if( cmd[0] == CMD_START ) {
      Rascii.write("Interface aktiv");
    }
    else if( cmd[0] == CMD_STOP ) {
      Rascii.write("Interface passiv");
    }
    else if( cmd[0] == CMD_DOUT ) {
      //digitalOut(cmd);
    }
    else if( cmd[0] == CMD_AOUT ) {
      //analogOut(cmd);
    }
    else if( cmd[0] == CMD_GETCONF ) {
      //getConfig();
    }
    else if( cmd[0] == CMD_SETCONF ) {
      //setConfig(cmd);
    }
    else if( cmd[0] == CMD_RESETCONF ) {
      //resetConfig();
    }
    else if( cmd[0] == CMD_GETCV ) {
      //getCV(cmd);
    }
    else if( cmd[0] == CMD_SETCV ) {
      setCV(cmd);
    }
    else {
      sprintf(s, "Befehl nicht erkannt: %02X", cmd[0]);
      Rascii.write(s);
    }
  //}
}
void CAN_AUSWERTUNG (void){
  k = id >> 8;
    if((len == 3) &&
               (buf[0] == 0xFF) &&
               (buf[1] == 0xFF) &&
               (buf[2] == 0xFF)){
          ANMELDEN();
               }
    switch(k)
    {
      case 0x7://SYSTEMNACHRICHT
        FBSYS();
      break;
      case 0x6:
      break;
      case 0x5://RFID-READER
          FB500();
      break;
      case 0x4://RÜCKMELDER
          FB400();
      break;
      case 0x3:
      break;
      case 0x2:
      break;
      case 0x1:
      break;
    }
}
void FB400 (void){                                        // RÜCKMELDER
  byte din[3]; 
  din[0] = id&0xFF; // address
  din[1] = buf[0] & 0x7F; // port
  if(buf[0]&0x80){din[2] = 1;}
  else{din[2] = 0;}
  Rascii.write(EVT_DSEN, din, 3, EVT);
  return;
}
void FB500 (void){ //RFID-READER
  byte din[6]; 
  din[0] = id&0xFF; // address
  din[1] = 0x01; // port
  din[2] = buf[0];
  din[3] = buf[1];
  din[4] = buf[2];
  din[5] = buf[3];
  Rascii.write(EVT_RFID, din, 6, EVT);
  return;
}
void FBSYS (void){  //SYSTEMNACHRICHT
  byte din[6]; 
  din[0] = id&0xFF;                                      // Adresse
  din[1] = buf[0];                                       // HW-Version
  din[2] = buf[1];                                       // SW-Vers
  din[3] = buf[2];                                       // Meldertyp
  Rascii.write(EVT_PRGM, din, 4, EVT);
  return;
}
//----INITIALISIERUNG----
void setup() {

Rascii.begin(9600);

START_INIT:
    if(CAN_OK == CAN.begin(CAN_125KBPS))                   // MCP CAN öffnen bei 125 kbps
    {                                                     // irgendeine Ausgabe vlt. LED
    }
    else
    {
        delay(100);                                        // wenn der Controller nicht reagiert
        goto START_INIT;                                   // nochmal versuchen - und irgendeine andere Ausgabe
    }
}
//----HAUPTPROGRAMM----
void loop() {
  byte cmd[64];
  
  if( Rascii.read(cmd) ) {evaluateCmd(cmd);}
  
  if(CAN_MSGAVAIL == CAN.checkReceive()){
    CAN.readMsgBufID(&id,&len,buf);                       //Wenn gültige CAN-Msg erhalten, auslesen und verarbeiten:
    CAN_AUSWERTUNG();
    }
}
